﻿using System;
using System.Security.Permissions;

[assembly: SecurityPermission(SecurityAction.RequestMinimum)]
namespace DateCalculator.Library
{
    /// <summary>
    /// Class to provide different functionalities to work with date calculations.
    /// </summary>
    public static class Calculator
    {
        /// <summary>
        /// Calculates the difference.
        /// </summary>
        /// <param name="firstDate">The first date.</param>
        /// <param name="secondDate">The second date.</param>
        /// <returns>Difference between supplied dates.</returns>
        public static DateSpan CalculateDifference(DateTime firstDate, DateTime secondDate)
        {
            var years = CalculateYears(firstDate, secondDate);
            var months = CalculateMonths(firstDate, secondDate, years);
            var days = CalculateDays(firstDate, secondDate, years, months);

            var result = new DateSpan
                             {
                                 Years = years,
                                 Months = months,
                                 Days = days
                             };

            return result;
        }

        /// <summary>
        /// Gets the modified date.
        /// </summary>
        /// <param name="fromDate">From date.</param>
        /// <param name="function">The function.</param>
        /// <param name="dateOffset">The date offset.</param>
        /// <returns>Result of either addition or subtraction to/from a supplied date.</returns>
        public static DateTime GetModifiedDate(DateTime fromDate, DateFunction function, DateSpan dateOffset)
        {
            return function == DateFunction.Add ? AddToDate(fromDate, dateOffset) : SubtractFromDate(fromDate, dateOffset);
        }


        /// <summary>
        /// Adds to date.
        /// </summary>
        /// <param name="fromDate">From date.</param>
        /// <param name="dateOffset">The date offset.</param>
        /// <returns>Result of addition to a supplied date.</returns>
        private static DateTime AddToDate(DateTime fromDate, DateSpan dateOffset)
        {
            return fromDate.AddYears(dateOffset.Years).AddMonths(dateOffset.Months).AddDays(dateOffset.Days);
        }

        /// <summary>
        /// Subtracts from date.
        /// </summary>
        /// <param name="fromDate">From date.</param>
        /// <param name="dateOffset">The date offset.</param>
        /// <returns>Result of subtraction from a supplied date.</returns>
        private static DateTime SubtractFromDate(DateTime fromDate, DateSpan dateOffset)
        {
            return fromDate.AddDays(-dateOffset.Days).AddMonths(-dateOffset.Months).AddYears(-dateOffset.Years);
        }

        /// <summary>
        /// Calculates the difference in no. of days.
        /// </summary>
        /// <param name="firstDate">The first date.</param>
        /// <param name="secondDate">The second date.</param>
        /// <param name="years">The years.</param>
        /// <param name="months">The months.</param>
        /// <returns>No. of days after subtraction from one date and the other date.</returns>
        private static int CalculateDays(DateTime firstDate, DateTime secondDate, int years, int months)
        {
            return secondDate.Subtract(firstDate.AddYears(years).AddMonths(months)).Days;
        }

        /// <summary>
        /// Calculates the difference in no. of months.
        /// </summary>
        /// <param name="firstDate">The first date.</param>
        /// <param name="secondDate">The second date.</param>
        /// <param name="years">The years.</param>
        /// <returns>No. of months after subtraction from one date and the other date.</returns>
        private static int CalculateMonths(DateTime firstDate, DateTime secondDate, int years)
        {
            var result = 0;

            for (var i = 1; i <= 12; i++)
            {
                if (secondDate.Subtract(firstDate.AddYears(years).AddMonths(i)).Days >= 0)
                {
                    result = i;
                }
                else
                {
                    break;
                }
            }

            return result;
        }

        /// <summary>
        /// Calculates the difference in no. of years.
        /// </summary>
        /// <param name="firstDate">The first date.</param>
        /// <param name="secondDate">The second date.</param>
        /// <returns>No. of years after subtraction from one date and the other date.</returns>
        private static int CalculateYears(DateTime firstDate, DateTime secondDate)
        {
            if (secondDate.Month < firstDate.Month ||
                secondDate.Month == firstDate.Month &&
                secondDate.Day < firstDate.Day)
            {
                return ((secondDate.Year - firstDate.Year) - 1);
            }

            return (secondDate.Year - firstDate.Year);
        }
    }
}