﻿using System;
using System.Windows.Forms;
using DateCalculator.Library;

namespace DateCalculator.UI
{
    public partial class MainForm : Form
    {
        private const string CalculateAgeOrExperienceTab = "DifferenceTab";
        private const string AddSubtractDateTab = "AddSubtractDate";

        /// <summary>
        /// Initializes a new instance of the <see cref="MainForm"/> class.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Handles the click event for Calculate button in "Age/Experience" mode.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void CalculateButtonClick(object sender, EventArgs e)
        {
            CalculateAgeOrExperience();
        }

        /// <summary>
        /// Calculates the age or experience and binds the results of calculation to user control.
        /// </summary>
        private void CalculateAgeOrExperience()
        {
            var firstInput = FirstDate.Value > SecondDate.Value ? SecondDate.Value : FirstDate.Value;
            var secondInput = FirstDate.Value > SecondDate.Value ? FirstDate.Value : SecondDate.Value;

            var difference = Calculator.CalculateDifference(firstInput, secondInput);

            ResultSummary.Results = difference;
            ResultSummary.Bind();
        }

        /// <summary>
        /// Handles the click event of Reset button.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ResetButtonClick(object sender, EventArgs e)
        {
            ResetAgeOrExperienceInputsAndOutputs();
        }

        /// <summary>
        /// Resets the age or experience inputs and outputs.
        /// </summary>
        private void ResetAgeOrExperienceInputsAndOutputs()
        {
            FirstDate.Value = SecondDate.Value = DateTime.Now;
            ResultSummary.Reset();
        }

        /// <summary>
        /// Handles the click event of Reset2 button.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Reset2ButtonClick(object sender, EventArgs e)
        {
            ResetAddSubtractDateInputsAndOutputs();
        }

        /// <summary>
        /// Resets the add subtract date inputs and outputs.
        /// </summary>
        private void ResetAddSubtractDateInputsAndOutputs()
        {
            FromDate.Value = DateTime.Today;
            Years.Value = Months.Value = Days.Value = 0;
            Add.Checked = true;
            ResultedDate.Clear();
        }

        /// <summary>
        /// Handles the click event of Calculate2 button.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Calculate2ButtonClick(object sender, EventArgs e)
        {
            CalculateResultsForAddOrSubtractDate();
        }

        /// <summary>
        /// Calculates the results for add or subtract date.
        /// </summary>
        private void CalculateResultsForAddOrSubtractDate()
        {
            var fromDate = FromDate.Value;

            var function = Add.Checked ? DateFunction.Add : DateFunction.Subtract;

            var dateSpan = new DateSpan
                               {
                                   Years = Convert.ToInt32(Years.Value),
                                   Months = Convert.ToInt32(Months.Value),
                                   Days = Convert.ToInt32(Days.Value)
                               };

            var result = Calculator.GetModifiedDate(fromDate, function, dateSpan);

            ResultedDate.Text = result.ToLongDateString();
        }

        /// <summary>
        /// Handles the click event of Exit Menu item.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ExitMenuClick(object sender, EventArgs e)
        {
            Close();
            Application.Exit();
        }

        /// <summary>
        /// Handles the click event of Reset Menu item.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ResetMenuClick(object sender, EventArgs e)
        {
            switch (ApplicationTabs.SelectedTab.Name)
            {
                case CalculateAgeOrExperienceTab:
                    ResetAgeOrExperienceInputsAndOutputs();
                    break;

                case AddSubtractDateTab:
                    ResetAddSubtractDateInputsAndOutputs();
                    break;
            }
        }

        /// <summary>
        /// Handles the click event of Calculate Menu item.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void CalculateMenuClick(object sender, EventArgs e)
        {
            switch (ApplicationTabs.SelectedTab.Name)
            {
                case CalculateAgeOrExperienceTab:
                    CalculateAgeOrExperience();
                    break;

                case AddSubtractDateTab:
                    CalculateResultsForAddOrSubtractDate();
                    break;
            }
        }

        /// <summary>
        /// Handles the click event of About Menu item.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AboutMenuClick(object sender, EventArgs e)
        {
            var aboutForm = new AboutDateCalculator();
            aboutForm.Show(this);
        }
    }
}